<?php

namespace Drupal\vereinonline_auth;

use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Exception\ConnectException;

/**
 * Vereinonline login common class.
 */
class VereinonlineCommon {

  /**
   * Vereinonline login form validation.
   */
  public static function userRegisterValidate($form, FormStateInterface &$form_state) {
    $config = \Drupal::configFactory()->get('vereinonline_auth.settings');

    if (strlen($config->get('api_url')) === 0
    || strlen($config->get('api_user')) === 0
    || strlen($config->get('api_password')) === 0) {
      // Vereinonline_auth not configured.
      \Drupal::logger('vereinonline_auth')->notice(t('vereinonline_auth is not configured.'));
      return;
    }

    /** @var \Drupal\user\Entity\User|bool $user */
    $user = user_load_by_name($form_state->getValue('name'));
    if ($user === FALSE
    || $user->hasRole($config->get('user_role'))) {
      // No local user or local user with Vereinonline user role.
      self::syncLogin($form_state);
    }
  }

  /**
   * Vereinonline login sync.
   */
  public static function syncLogin(FormStateInterface &$form_state) {
    $config = \Drupal::configFactory()->get('vereinonline_auth.settings');
    $response = self::checkLogin($form_state->getValue('name'), $form_state->getValue('pass'));
    if ($response->error !== ''
    && isset($response->errorcode)
    && $response->errorcode === 'noconnection') {
      // No connection to Vereinonline.
      if ($config->get('fallback_noconnection') === TRUE) {
        // Falling back to local.
        \Drupal::logger('vereinonline_auth')->warning(t('No connection to @server. Fallback to local user database.', ['@server' => $config->get('api_url')]));
        return;
      }
      else {
        // Failing, no fallback.
        \Drupal::logger('vereinonline_auth')->error(t('No connection to @server.', ['@server' => $config->get('api_url')]));
        $form_state->clearErrors();
        $form_state->setErrorByName('name', t('No connection to password server. Please try again later.'));
        return;
      }
    }
    elseif ($response->error !== ''
    && isset($response->errorcode)
    && $response->errorcode === 'requesterror') {
      \Drupal::logger('vereinonline_auth')->error(t('Request error with @server: @error', ['@server' => $config->get('api_url'), '@error' => $response->error]));
      $form_state->clearErrors();
      $form_state->setErrorByName('name', t('Request error with server. Please try again later.'));
      return;
    }
    elseif ($response->error !== '') {
      // Username/password not valid at Vereinonline.
      $form_state->clearErrors();
      $form_state->setErrorByName('name', t('Unrecognized username or password. <a href=":password">Forgot your password?</a>', [':password' => $config->get('api_url') . '?action=start_passwortvergessen']));
    }
    else {
      $form_state->clearErrors();
      $user = user_load_by_name($form_state->getValue('name'));
      if ($user === FALSE) {
        $user = User::create();
        \Drupal::logger('vereinonline_auth')->notice(t('New user created @username.', ['@username' => $form_state->getValue('name')]));
      }
      $user->setPassword($form_state->getValue('pass'));
      $user->setEmail($response->email);
      // This username must be unique and accept only a-Z,0-9, - _ @ .
      $user->setUsername($form_state->getValue('name'));
      $user->addRole($config->get('user_role'));
      $user->activate();
      // Save user account.
      $user->save();

      // Login as user.
      user_login_finalize($user);
      $form_state->set('uid', $user->id());
      $form_state->setValidationComplete(TRUE);
    }
  }

  /**
   * Vereinonline check login.
   */
  public static function checkLogin($name, $password) {
    if (drupal_valid_test_ua() !== FALSE) {
      // PHPunit test.
      $config = \Drupal::configFactory()->get('vereinonline_auth.settings');
      if ($config->get('api_url') === 'https://vereinonline_noconnection.org/') {
        $response = json_decode(file_get_contents(dirname(__FILE__) . '/../tests/src/Functional/Mocks/noconnection.json'));
      }
      elseif ($name === 'first.last' && $password === '7654321') {
        $response = json_decode(file_get_contents(dirname(__FILE__) . '/../tests/src/Functional/Mocks/richtigeruser.json'));
      }
      else {
        $response = json_decode(file_get_contents(dirname(__FILE__) . '/../tests/src/Functional/Mocks/fehlerhaft.json'));
      }
    }
    else {
      $response = self::request('CheckLogin', [
        'user' => $name,
        'password' => $password,
      ]);
    }
    return $response;
  }

  /**
   * Vereinonline request.
   */
  public static function request($command, $data) {
    $httpClient = \Drupal::httpClient();
    $config = \Drupal::configFactory()->get('vereinonline_auth.settings');
    $url = $config->get('api_url');
    $url .= '?json';
    $url .= '&function=' . $command;
    foreach ($data as $k => $v) {
      $url .= '&' . $k . '=' . urlencode($v);
    }
    $url .= '&token=A/' . $config->get('api_user') . '/' . md5($config->get('api_password'));
    try {
      $response = $httpClient->request('GET', $url, ['timeout' => 10])->getBody();
    }
    catch (ConnectException $e) {
      $response = '{ "error": "No connection to vereinonline.org server. Please try again later.", "errorcode": "noconnection" }';
    }
    catch (RequestException $e){
      $response = '{ "error": $e->getResponse(), "errorcode": "requesterror" }';
    }
    if ($response === '[""]') {
      $response = '{ "error": "Invalid response from vereinonline.org server. Please try again later.", "errorcode": "noconnection" }';
    }
    return json_decode($response);
  }

}
